@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from collections import OrderedDict
 from io import StringIO
 import re
@@ -15,6 +17,7 @@ from sqlalchemy import schema as sa_schema
 from sqlalchemy import sql
 from sqlalchemy import types as sqltypes
 from sqlalchemy.sql.elements import conv
+from sqlalchemy.sql.elements import quoted_name
 
 from .. import util
 from ..operations import ops
@@ -24,12 +27,10 @@ if TYPE_CHECKING:
     from typing import Literal
 
     from sqlalchemy.sql.elements import ColumnElement
-    from sqlalchemy.sql.elements import quoted_name
     from sqlalchemy.sql.elements import TextClause
     from sqlalchemy.sql.schema import CheckConstraint
     from sqlalchemy.sql.schema import Column
     from sqlalchemy.sql.schema import Constraint
-    from sqlalchemy.sql.schema import DefaultClause
     from sqlalchemy.sql.schema import FetchedValue
     from sqlalchemy.sql.schema import ForeignKey
     from sqlalchemy.sql.schema import ForeignKeyConstraint
@@ -52,13 +53,13 @@ MAX_PYTHON_ARGS = 255
 
 
 def _render_gen_name(
-    autogen_context: "AutogenContext",
-    name: Optional[Union["quoted_name", str]],
-) -> Optional[Union["quoted_name", str, "_f_name"]]:
+    autogen_context: AutogenContext,
+    name: sqla_compat._ConstraintName,
+) -> Optional[Union[quoted_name, str, _f_name]]:
     if isinstance(name, conv):
         return _f_name(_alembic_autogenerate_prefix(autogen_context), name)
     else:
-        return name
+        return sqla_compat.constraint_name_or_none(name)
 
 
 def _indent(text: str) -> str:
@@ -68,9 +69,9 @@ def _indent(text: str) -> str:
 
 
 def _render_python_into_templatevars(
-    autogen_context: "AutogenContext",
-    migration_script: "MigrationScript",
-    template_args: Dict[str, Union[str, "Config"]],
+    autogen_context: AutogenContext,
+    migration_script: MigrationScript,
+    template_args: Dict[str, Union[str, Config]],
 ) -> None:
     imports = autogen_context.imports
 
@@ -90,8 +91,8 @@ default_renderers = renderers = util.Dispatcher()
 
 
 def _render_cmd_body(
-    op_container: "ops.OpContainer",
-    autogen_context: "AutogenContext",
+    op_container: ops.OpContainer,
+    autogen_context: AutogenContext,
 ) -> str:
 
     buf = StringIO()
@@ -118,7 +119,7 @@ def _render_cmd_body(
 
 
 def render_op(
-    autogen_context: "AutogenContext", op: "ops.MigrateOperation"
+    autogen_context: AutogenContext, op: ops.MigrateOperation
 ) -> List[str]:
     renderer = renderers.dispatch(op)
     lines = util.to_list(renderer(autogen_context, op))
@@ -126,14 +127,14 @@ def render_op(
 
 
 def render_op_text(
-    autogen_context: "AutogenContext", op: "ops.MigrateOperation"
+    autogen_context: AutogenContext, op: ops.MigrateOperation
 ) -> str:
     return "\n".join(render_op(autogen_context, op))
 
 
 @renderers.dispatch_for(ops.ModifyTableOps)
 def _render_modify_table(
-    autogen_context: "AutogenContext", op: "ModifyTableOps"
+    autogen_context: AutogenContext, op: ModifyTableOps
 ) -> List[str]:
     opts = autogen_context.opts
     render_as_batch = opts.get("render_as_batch", False)
@@ -162,7 +163,7 @@ def _render_modify_table(
 
 @renderers.dispatch_for(ops.CreateTableCommentOp)
 def _render_create_table_comment(
-    autogen_context: "AutogenContext", op: "ops.CreateTableCommentOp"
+    autogen_context: AutogenContext, op: ops.CreateTableCommentOp
 ) -> str:
 
     templ = (
@@ -187,7 +188,7 @@ def _render_create_table_comment(
 
 @renderers.dispatch_for(ops.DropTableCommentOp)
 def _render_drop_table_comment(
-    autogen_context: "AutogenContext", op: "ops.DropTableCommentOp"
+    autogen_context: AutogenContext, op: ops.DropTableCommentOp
 ) -> str:
 
     templ = (
@@ -209,9 +210,7 @@ def _render_drop_table_comment(
 
 
 @renderers.dispatch_for(ops.CreateTableOp)
-def _add_table(
-    autogen_context: "AutogenContext", op: "ops.CreateTableOp"
-) -> str:
+def _add_table(autogen_context: AutogenContext, op: ops.CreateTableOp) -> str:
     table = op.to_table()
 
     args = [
@@ -261,9 +260,7 @@ def _add_table(
 
 
 @renderers.dispatch_for(ops.DropTableOp)
-def _drop_table(
-    autogen_context: "AutogenContext", op: "ops.DropTableOp"
-) -> str:
+def _drop_table(autogen_context: AutogenContext, op: ops.DropTableOp) -> str:
     text = "%(prefix)sdrop_table(%(tname)r" % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
         "tname": _ident(op.table_name),
@@ -275,9 +272,7 @@ def _drop_table(
 
 
 @renderers.dispatch_for(ops.CreateIndexOp)
-def _add_index(
-    autogen_context: "AutogenContext", op: "ops.CreateIndexOp"
-) -> str:
+def _add_index(autogen_context: AutogenContext, op: ops.CreateIndexOp) -> str:
     index = op.to_index()
 
     has_batch = autogen_context._has_batch
@@ -322,9 +317,7 @@ def _add_index(
 
 
 @renderers.dispatch_for(ops.DropIndexOp)
-def _drop_index(
-    autogen_context: "AutogenContext", op: "ops.DropIndexOp"
-) -> str:
+def _drop_index(autogen_context: AutogenContext, op: ops.DropIndexOp) -> str:
     index = op.to_index()
 
     has_batch = autogen_context._has_batch
@@ -360,14 +353,14 @@ def _drop_index(
 
 @renderers.dispatch_for(ops.CreateUniqueConstraintOp)
 def _add_unique_constraint(
-    autogen_context: "AutogenContext", op: "ops.CreateUniqueConstraintOp"
+    autogen_context: AutogenContext, op: ops.CreateUniqueConstraintOp
 ) -> List[str]:
     return [_uq_constraint(op.to_constraint(), autogen_context, True)]
 
 
 @renderers.dispatch_for(ops.CreateForeignKeyOp)
 def _add_fk_constraint(
-    autogen_context: "AutogenContext", op: "ops.CreateForeignKeyOp"
+    autogen_context: AutogenContext, op: ops.CreateForeignKeyOp
 ) -> str:
 
     args = [repr(_render_gen_name(autogen_context, op.constraint_name))]
@@ -416,7 +409,7 @@ def _add_check_constraint(constraint, autogen_context):
 
 @renderers.dispatch_for(ops.DropConstraintOp)
 def _drop_constraint(
-    autogen_context: "AutogenContext", op: "ops.DropConstraintOp"
+    autogen_context: AutogenContext, op: ops.DropConstraintOp
 ) -> str:
 
     if autogen_context._has_batch:
@@ -438,9 +431,7 @@ def _drop_constraint(
 
 
 @renderers.dispatch_for(ops.AddColumnOp)
-def _add_column(
-    autogen_context: "AutogenContext", op: "ops.AddColumnOp"
-) -> str:
+def _add_column(autogen_context: AutogenContext, op: ops.AddColumnOp) -> str:
 
     schema, tname, column = op.schema, op.table_name, op.column
     if autogen_context._has_batch:
@@ -460,9 +451,7 @@ def _add_column(
 
 
 @renderers.dispatch_for(ops.DropColumnOp)
-def _drop_column(
-    autogen_context: "AutogenContext", op: "ops.DropColumnOp"
-) -> str:
+def _drop_column(autogen_context: AutogenContext, op: ops.DropColumnOp) -> str:
 
     schema, tname, column_name = op.schema, op.table_name, op.column_name
 
@@ -485,7 +474,7 @@ def _drop_column(
 
 @renderers.dispatch_for(ops.AlterColumnOp)
 def _alter_column(
-    autogen_context: "AutogenContext", op: "ops.AlterColumnOp"
+    autogen_context: AutogenContext, op: ops.AlterColumnOp
 ) -> str:
 
     tname = op.table_name
@@ -554,7 +543,7 @@ class _f_name:
         return "%sf(%r)" % (self.prefix, _ident(self.name))
 
 
-def _ident(name: Optional[Union["quoted_name", str]]) -> Optional[str]:
+def _ident(name: Optional[Union[quoted_name, str]]) -> Optional[str]:
     """produce a __repr__() object for a string identifier that may
     use quoted_name() in SQLAlchemy 0.9 and greater.
 
@@ -564,7 +553,7 @@ def _ident(name: Optional[Union["quoted_name", str]]) -> Optional[str]:
     """
     if name is None:
         return name
-    elif isinstance(name, sql.elements.quoted_name):
+    elif isinstance(name, quoted_name):
         return str(name)
     elif isinstance(name, str):
         return name
@@ -572,7 +561,7 @@ def _ident(name: Optional[Union["quoted_name", str]]) -> Optional[str]:
 
 def _render_potential_expr(
     value: Any,
-    autogen_context: "AutogenContext",
+    autogen_context: AutogenContext,
     wrap_in_text: bool = True,
     is_server_default: bool = False,
 ) -> str:
@@ -595,7 +584,7 @@ def _render_potential_expr(
 
 
 def _get_index_rendered_expressions(
-    idx: "Index", autogen_context: "AutogenContext"
+    idx: Index, autogen_context: AutogenContext
 ) -> List[str]:
     return [
         repr(_ident(getattr(exp, "name", None)))
@@ -606,8 +595,8 @@ def _get_index_rendered_expressions(
 
 
 def _uq_constraint(
-    constraint: "UniqueConstraint",
-    autogen_context: "AutogenContext",
+    constraint: UniqueConstraint,
+    autogen_context: AutogenContext,
     alter: bool,
 ) -> str:
     opts: List[Tuple[str, Any]] = []
@@ -652,11 +641,11 @@ def _user_autogenerate_prefix(autogen_context, target):
         return prefix
 
 
-def _sqlalchemy_autogenerate_prefix(autogen_context: "AutogenContext") -> str:
+def _sqlalchemy_autogenerate_prefix(autogen_context: AutogenContext) -> str:
     return autogen_context.opts["sqlalchemy_module_prefix"] or ""
 
 
-def _alembic_autogenerate_prefix(autogen_context: "AutogenContext") -> str:
+def _alembic_autogenerate_prefix(autogen_context: AutogenContext) -> str:
     if autogen_context._has_batch:
         return "batch_op."
     else:
@@ -664,8 +653,8 @@ def _alembic_autogenerate_prefix(autogen_context: "AutogenContext") -> str:
 
 
 def _user_defined_render(
-    type_: str, object_: Any, autogen_context: "AutogenContext"
-) -> Union[str, "Literal[False]"]:
+    type_: str, object_: Any, autogen_context: AutogenContext
+) -> Union[str, Literal[False]]:
     if "render_item" in autogen_context.opts:
         render = autogen_context.opts["render_item"]
         if render:
@@ -675,7 +664,7 @@ def _user_defined_render(
     return False
 
 
-def _render_column(column: "Column", autogen_context: "AutogenContext") -> str:
+def _render_column(column: Column, autogen_context: AutogenContext) -> str:
     rendered = _user_defined_render("column", column, autogen_context)
     if rendered is not False:
         return rendered
@@ -731,19 +720,15 @@ def _render_column(column: "Column", autogen_context: "AutogenContext") -> str:
     }
 
 
-def _should_render_server_default_positionally(
-    server_default: Union["Computed", "DefaultClause"]
-) -> bool:
+def _should_render_server_default_positionally(server_default: Any) -> bool:
     return sqla_compat._server_default_is_computed(
         server_default
     ) or sqla_compat._server_default_is_identity(server_default)
 
 
 def _render_server_default(
-    default: Optional[
-        Union["FetchedValue", str, "TextClause", "ColumnElement"]
-    ],
-    autogen_context: "AutogenContext",
+    default: Optional[Union[FetchedValue, str, TextClause, ColumnElement]],
+    autogen_context: AutogenContext,
     repr_: bool = True,
 ) -> Optional[str]:
     rendered = _user_defined_render("server_default", default, autogen_context)
@@ -769,7 +754,7 @@ def _render_server_default(
 
 
 def _render_computed(
-    computed: "Computed", autogen_context: "AutogenContext"
+    computed: Computed, autogen_context: AutogenContext
 ) -> str:
     text = _render_potential_expr(
         computed.sqltext, autogen_context, wrap_in_text=False
@@ -786,7 +771,7 @@ def _render_computed(
 
 
 def _render_identity(
-    identity: "Identity", autogen_context: "AutogenContext"
+    identity: Identity, autogen_context: AutogenContext
 ) -> str:
     # always=None means something different than always=False
     kwargs = OrderedDict(always=identity.always)
@@ -800,7 +785,7 @@ def _render_identity(
     }
 
 
-def _get_identity_options(identity_options: "Identity") -> OrderedDict:
+def _get_identity_options(identity_options: Identity) -> OrderedDict:
     kwargs = OrderedDict()
     for attr in sqla_compat._identity_options_attrs:
         value = getattr(identity_options, attr, None)
@@ -810,8 +795,8 @@ def _get_identity_options(identity_options: "Identity") -> OrderedDict:
 
 
 def _repr_type(
-    type_: "TypeEngine",
-    autogen_context: "AutogenContext",
+    type_: TypeEngine,
+    autogen_context: AutogenContext,
     _skip_variants: bool = False,
 ) -> str:
     rendered = _user_defined_render("type", type_, autogen_context)
@@ -839,9 +824,9 @@ def _repr_type(
             return "%s.%r" % (dname, type_)
     elif impl_rt:
         return impl_rt
+    elif not _skip_variants and sqla_compat._type_has_variants(type_):
+        return _render_Variant_type(type_, autogen_context)
     elif mod.startswith("sqlalchemy."):
-        if not _skip_variants and sqla_compat._type_has_variants(type_):
-            return _render_Variant_type(type_, autogen_context)
         if "_render_%s_type" % type_.__visit_name__ in globals():
             fn = globals()["_render_%s_type" % type_.__visit_name__]
             return fn(type_, autogen_context)
@@ -853,9 +838,7 @@ def _repr_type(
         return "%s%r" % (prefix, type_)
 
 
-def _render_ARRAY_type(
-    type_: "ARRAY", autogen_context: "AutogenContext"
-) -> str:
+def _render_ARRAY_type(type_: ARRAY, autogen_context: AutogenContext) -> str:
     return cast(
         str,
         _render_type_w_subtype(
@@ -865,7 +848,7 @@ def _render_ARRAY_type(
 
 
 def _render_Variant_type(
-    type_: "TypeEngine", autogen_context: "AutogenContext"
+    type_: TypeEngine, autogen_context: AutogenContext
 ) -> str:
     base_type, variant_mapping = sqla_compat._get_variant_mapping(type_)
     base = _repr_type(base_type, autogen_context, _skip_variants=True)
@@ -880,12 +863,12 @@ def _render_Variant_type(
 
 
 def _render_type_w_subtype(
-    type_: "TypeEngine",
-    autogen_context: "AutogenContext",
+    type_: TypeEngine,
+    autogen_context: AutogenContext,
     attrname: str,
     regexp: str,
     prefix: Optional[str] = None,
-) -> Union[Optional[str], "Literal[False]"]:
+) -> Union[Optional[str], Literal[False]]:
     outer_repr = repr(type_)
     inner_type = getattr(type_, attrname, None)
     if inner_type is None:
@@ -917,9 +900,9 @@ _constraint_renderers = util.Dispatcher()
 
 
 def _render_constraint(
-    constraint: "Constraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: Optional["MetaData"],
+    constraint: Constraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: Optional[MetaData],
 ) -> Optional[str]:
     try:
         renderer = _constraint_renderers.dispatch(constraint)
@@ -932,9 +915,9 @@ def _render_constraint(
 
 @_constraint_renderers.dispatch_for(sa_schema.PrimaryKeyConstraint)
 def _render_primary_key(
-    constraint: "PrimaryKeyConstraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: Optional["MetaData"],
+    constraint: PrimaryKeyConstraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: Optional[MetaData],
 ) -> Optional[str]:
     rendered = _user_defined_render("primary_key", constraint, autogen_context)
     if rendered is not False:
@@ -958,9 +941,9 @@ def _render_primary_key(
 
 
 def _fk_colspec(
-    fk: "ForeignKey",
+    fk: ForeignKey,
     metadata_schema: Optional[str],
-    namespace_metadata: "MetaData",
+    namespace_metadata: MetaData,
 ) -> str:
     """Implement a 'safe' version of ForeignKey._get_colspec() that
     won't fail if the remote table can't be resolved.
@@ -987,7 +970,7 @@ def _fk_colspec(
         if table_fullname in namespace_metadata.tables:
             col = namespace_metadata.tables[table_fullname].c.get(colname)
             if col is not None:
-                colname = _ident(col.name)
+                colname = _ident(col.name)  # type: ignore[assignment]
 
     colspec = "%s.%s" % (table_fullname, colname)
 
@@ -995,7 +978,7 @@ def _fk_colspec(
 
 
 def _populate_render_fk_opts(
-    constraint: "ForeignKeyConstraint", opts: List[Tuple[str, str]]
+    constraint: ForeignKeyConstraint, opts: List[Tuple[str, str]]
 ) -> None:
 
     if constraint.onupdate:
@@ -1012,9 +995,9 @@ def _populate_render_fk_opts(
 
 @_constraint_renderers.dispatch_for(sa_schema.ForeignKeyConstraint)
 def _render_foreign_key(
-    constraint: "ForeignKeyConstraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: "MetaData",
+    constraint: ForeignKeyConstraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: MetaData,
 ) -> Optional[str]:
     rendered = _user_defined_render("foreign_key", constraint, autogen_context)
     if rendered is not False:
@@ -1051,9 +1034,9 @@ def _render_foreign_key(
 
 @_constraint_renderers.dispatch_for(sa_schema.UniqueConstraint)
 def _render_unique_constraint(
-    constraint: "UniqueConstraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: Optional["MetaData"],
+    constraint: UniqueConstraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: Optional[MetaData],
 ) -> str:
     rendered = _user_defined_render("unique", constraint, autogen_context)
     if rendered is not False:
@@ -1064,9 +1047,9 @@ def _render_unique_constraint(
 
 @_constraint_renderers.dispatch_for(sa_schema.CheckConstraint)
 def _render_check_constraint(
-    constraint: "CheckConstraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: Optional["MetaData"],
+    constraint: CheckConstraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: Optional[MetaData],
 ) -> Optional[str]:
     rendered = _user_defined_render("check", constraint, autogen_context)
     if rendered is not False:
@@ -1104,9 +1087,7 @@ def _render_check_constraint(
 
 
 @renderers.dispatch_for(ops.ExecuteSQLOp)
-def _execute_sql(
-    autogen_context: "AutogenContext", op: "ops.ExecuteSQLOp"
-) -> str:
+def _execute_sql(autogen_context: AutogenContext, op: ops.ExecuteSQLOp) -> str:
     if not isinstance(op.sqltext, str):
         raise NotImplementedError(
             "Autogenerate rendering of SQL Expression language constructs "
