@@ -306,9 +306,8 @@ def _exec_drop_col_constraint(
     )
     # from http://www.mssqltips.com/sqlservertip/1425/\
     # working-with-default-constraints-in-sql-server/
-    # TODO: needs table formatting, etc.
     return """declare @const_name varchar(256)
-select @const_name = [name] from %(type)s
+select @const_name = QUOTENAME([name]) from %(type)s
 where parent_object_id = object_id('%(schema_dot)s%(tname)s')
 and col_name(parent_object_id, parent_column_id) = '%(colname)s'
 exec('alter table %(tname_quoted)s drop constraint ' + @const_name)""" % {
@@ -327,7 +326,7 @@ def _exec_drop_col_fk_constraint(
     schema, tname, colname = element.schema, element.tname, element.colname
 
     return """declare @const_name varchar(256)
-select @const_name = [name] from
+select @const_name = QUOTENAME([name]) from
 sys.foreign_keys fk join sys.foreign_key_columns fkc
 on fk.object_id=fkc.constraint_object_id
 where fkc.parent_object_id = object_id('%(schema_dot)s%(tname)s')
