@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from typing import Any
 from typing import Callable
 from typing import ContextManager
 from typing import Dict
@@ -11,6 +12,7 @@ from typing import Tuple
 from typing import TYPE_CHECKING
 from typing import Union
 
+from .migration import _ProxyTransaction
 from .migration import MigrationContext
 from .. import util
 from ..operations import Operations
@@ -18,15 +20,22 @@ from ..operations import Operations
 if TYPE_CHECKING:
     from typing import Literal
 
+    from sqlalchemy.engine import URL
     from sqlalchemy.engine.base import Connection
+    from sqlalchemy.sql.elements import ClauseElement
     from sqlalchemy.sql.schema import MetaData
 
-    from .migration import _ProxyTransaction
     from ..config import Config
+    from ..ddl import DefaultImpl
+    from ..operations.ops import MigrateOperation
     from ..script.base import ScriptDirectory
 
 _RevNumber = Optional[Union[str, Tuple[str, ...]]]
 
+ProcessRevisionDirectiveFn = Callable[
+    [MigrationContext, Tuple[str, str], List["MigrateOperation"]], None
+]
+
 
 class EnvironmentContext(util.ModuleClsProxy):
 
@@ -91,14 +100,14 @@ class EnvironmentContext(util.ModuleClsProxy):
 
     """
 
-    _migration_context: Optional["MigrationContext"] = None
+    _migration_context: Optional[MigrationContext] = None
 
-    config: "Config" = None  # type:ignore[assignment]
+    config: Config = None  # type:ignore[assignment]
     """An instance of :class:`.Config` representing the
     configuration file contents as well as other variables
     set programmatically within it."""
 
-    script: "ScriptDirectory" = None  # type:ignore[assignment]
+    script: ScriptDirectory = None  # type:ignore[assignment]
     """An instance of :class:`.ScriptDirectory` which provides
     programmatic access to version files within the ``versions/``
     directory.
@@ -106,7 +115,7 @@ class EnvironmentContext(util.ModuleClsProxy):
     """
 
     def __init__(
-        self, config: "Config", script: "ScriptDirectory", **kw
+        self, config: Config, script: ScriptDirectory, **kw: Any
     ) -> None:
         r"""Construct a new :class:`.EnvironmentContext`.
 
@@ -121,7 +130,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         self.script = script
         self.context_opts = kw
 
-    def __enter__(self) -> "EnvironmentContext":
+    def __enter__(self) -> EnvironmentContext:
         """Establish a context which provides a
         :class:`.EnvironmentContext` object to
         env.py scripts.
@@ -133,7 +142,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         self._install_proxy()
         return self
 
-    def __exit__(self, *arg, **kw) -> None:
+    def __exit__(self, *arg: Any, **kw: Any) -> None:
         self._remove_proxy()
 
     def is_offline_mode(self) -> bool:
@@ -261,18 +270,22 @@ class EnvironmentContext(util.ModuleClsProxy):
         return self.context_opts.get("tag", None)
 
     @overload
-    def get_x_argument(  # type:ignore[misc]
-        self, as_dictionary: "Literal[False]" = ...
-    ) -> List[str]:
+    def get_x_argument(self, as_dictionary: Literal[False]) -> List[str]:
         ...
 
     @overload
-    def get_x_argument(  # type:ignore[misc]
-        self, as_dictionary: "Literal[True]" = ...
-    ) -> Dict[str, str]:
+    def get_x_argument(self, as_dictionary: Literal[True]) -> Dict[str, str]:
         ...
 
-    def get_x_argument(self, as_dictionary: bool = False):
+    @overload
+    def get_x_argument(
+        self, as_dictionary: bool = ...
+    ) -> Union[List[str], Dict[str, str]]:
+        ...
+
+    def get_x_argument(
+        self, as_dictionary: bool = False
+    ) -> Union[List[str], Dict[str, str]]:
         """Return the value(s) passed for the ``-x`` argument, if any.
 
         The ``-x`` argument is an open ended flag that allows any user-defined
@@ -321,33 +334,35 @@ class EnvironmentContext(util.ModuleClsProxy):
 
     def configure(
         self,
-        connection: Optional["Connection"] = None,
-        url: Optional[str] = None,
+        connection: Optional[Connection] = None,
+        url: Optional[Union[str, URL]] = None,
         dialect_name: Optional[str] = None,
-        dialect_opts: Optional[dict] = None,
+        dialect_opts: Optional[Dict[str, Any]] = None,
         transactional_ddl: Optional[bool] = None,
         transaction_per_migration: bool = False,
         output_buffer: Optional[TextIO] = None,
         starting_rev: Optional[str] = None,
         tag: Optional[str] = None,
-        template_args: Optional[dict] = None,
+        template_args: Optional[Dict[str, Any]] = None,
         render_as_batch: bool = False,
-        target_metadata: Optional["MetaData"] = None,
-        include_name: Optional[Callable] = None,
-        include_object: Optional[Callable] = None,
+        target_metadata: Optional[MetaData] = None,
+        include_name: Optional[Callable[..., bool]] = None,
+        include_object: Optional[Callable[..., bool]] = None,
         include_schemas: bool = False,
-        process_revision_directives: Optional[Callable] = None,
+        process_revision_directives: Optional[
+            ProcessRevisionDirectiveFn
+        ] = None,
         compare_type: bool = False,
         compare_server_default: bool = False,
-        render_item: Optional[Callable] = None,
+        render_item: Optional[Callable[..., bool]] = None,
         literal_binds: bool = False,
         upgrade_token: str = "upgrades",
         downgrade_token: str = "downgrades",
         alembic_module_prefix: str = "op.",
         sqlalchemy_module_prefix: str = "sa.",
         user_module_prefix: Optional[str] = None,
-        on_version_apply: Optional[Callable] = None,
-        **kw,
+        on_version_apply: Optional[Callable[..., None]] = None,
+        **kw: Any,
     ) -> None:
         """Configure a :class:`.MigrationContext` within this
         :class:`.EnvironmentContext` which will provide database
@@ -828,7 +843,7 @@ class EnvironmentContext(util.ModuleClsProxy):
             opts=opts,
         )
 
-    def run_migrations(self, **kw) -> None:
+    def run_migrations(self, **kw: Any) -> None:
         """Run migrations as determined by the current command line
         configuration
         as well as versioning information present (or not) in the current
@@ -852,7 +867,11 @@ class EnvironmentContext(util.ModuleClsProxy):
         with Operations.context(self._migration_context):
             self.get_context().run_migrations(**kw)
 
-    def execute(self, sql, execution_options=None):
+    def execute(
+        self,
+        sql: Union[ClauseElement, str],
+        execution_options: Optional[dict] = None,
+    ) -> None:
         """Execute the given SQL using the current change context.
 
         The behavior of :meth:`.execute` is the same
@@ -866,7 +885,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         """
         self.get_context().execute(sql, execution_options=execution_options)
 
-    def static_output(self, text):
+    def static_output(self, text: str) -> None:
         """Emit text directly to the "offline" SQL stream.
 
         Typically this is for emitting comments that
@@ -879,7 +898,7 @@ class EnvironmentContext(util.ModuleClsProxy):
 
     def begin_transaction(
         self,
-    ) -> Union["_ProxyTransaction", ContextManager]:
+    ) -> Union[_ProxyTransaction, ContextManager[None]]:
         """Return a context manager that will
         enclose an operation within a "transaction",
         as defined by the environment's offline
@@ -925,7 +944,7 @@ class EnvironmentContext(util.ModuleClsProxy):
 
         return self.get_context().begin_transaction()
 
-    def get_context(self) -> "MigrationContext":
+    def get_context(self) -> MigrationContext:
         """Return the current :class:`.MigrationContext` object.
 
         If :meth:`.EnvironmentContext.configure` has not been
@@ -937,7 +956,7 @@ class EnvironmentContext(util.ModuleClsProxy):
             raise Exception("No context has been configured yet.")
         return self._migration_context
 
-    def get_bind(self):
+    def get_bind(self) -> Connection:
         """Return the current 'bind'.
 
         In "online" mode, this is the
@@ -948,7 +967,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         has first been made available via :meth:`.configure`.
 
         """
-        return self.get_context().bind
+        return self.get_context().bind  # type: ignore[return-value]
 
-    def get_impl(self):
+    def get_impl(self) -> DefaultImpl:
         return self.get_context().impl
