@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 from contextlib import contextmanager
+from contextlib import nullcontext
 import logging
 import sys
 from typing import Any
@@ -33,6 +34,7 @@ from ..util.compat import EncodedIO
 
 if TYPE_CHECKING:
     from sqlalchemy.engine import Dialect
+    from sqlalchemy.engine import URL
     from sqlalchemy.engine.base import Connection
     from sqlalchemy.engine.base import Transaction
     from sqlalchemy.engine.mock import MockConnection
@@ -216,7 +218,7 @@ class MigrationContext:
     def configure(
         cls,
         connection: Optional[Connection] = None,
-        url: Optional[str] = None,
+        url: Optional[Union[str, URL]] = None,
         dialect_name: Optional[str] = None,
         dialect: Optional[Dialect] = None,
         environment_context: Optional[EnvironmentContext] = None,
@@ -366,7 +368,7 @@ class MigrationContext:
 
     def begin_transaction(
         self, _per_migration: bool = False
-    ) -> Union[_ProxyTransaction, ContextManager]:
+    ) -> Union[_ProxyTransaction, ContextManager[None]]:
         """Begin a logical transaction for migration operations.
 
         This method is used within an ``env.py`` script to demarcate where
@@ -408,12 +410,8 @@ class MigrationContext:
 
         """
 
-        @contextmanager
-        def do_nothing():
-            yield
-
         if self._in_external_transaction:
-            return do_nothing()
+            return nullcontext()
 
         if self.impl.transactional_ddl:
             transaction_now = _per_migration == self._transaction_per_migration
@@ -421,13 +419,13 @@ class MigrationContext:
             transaction_now = _per_migration is True
 
         if not transaction_now:
-            return do_nothing()
+            return nullcontext()
 
         elif not self.impl.transactional_ddl:
             assert _per_migration
 
             if self.as_sql:
-                return do_nothing()
+                return nullcontext()
             else:
                 # track our own notion of a "transaction block", which must be
                 # committed when complete.   Don't rely upon whether or not the
@@ -443,7 +441,7 @@ class MigrationContext:
                 in_transaction = self._transaction is not None
 
                 if in_transaction:
-                    return do_nothing()
+                    return nullcontext()
                 else:
                     assert self.connection is not None
                     self._transaction = (
